﻿////////////////////////////////////////////////////////////////////////////////
//
//  TeraPad ユニット Ver.0.18
//
////////////////////////////////////////////////////////////////////////////////

unit TeraPad;

interface

uses
  Windows,
  Messages,
  SysUtils;

const
  MAX_GETTEXT_SIZE    = 65534;          // WM_GETTEXT で取得する最大テキスト

  TP_OUTPUT_OFF = 0;
  TP_OUTPUT_ON  = 1;

  TP_TEXT_ALL         = 0;
  TP_TEXT_SEL         = 1;
  TP_TEXT_LINES       = 2;
  TP_MAP_PREFIX       = 'TeraPad_Map_'; // マッピングファイルオブジェクトの名前

  TP_NUM_ROW          = 0;              // 描画される１行ごとの行番号
  TP_NUM_LINE         = 1;              // 折り返し表示されている１行文字列に対応した行番号(論理行)

  TPMSG_ROWJUMP       = WM_USER + 100;  // 行移動
  TPMSG_ROWJUNP       = TPMSG_ROWJUMP;  // 行移動(以前のスペルミスバージョンとの互換性を保つために…f(^^;))
  TPMSG_READINI       = WM_USER + 101;  // INIを読み込んで反映
  TPMSG_ENDJUNP       = WM_USER + 102;  // 文末移動
  TPMSG_SAVEFILE      = WM_APP  + 400;  // 上書き保存を実行
  TPMSG_LOADFILE      = WM_APP  + 401;  // TeraPadにファイルを読み込ませる
  TPMSG_GETACTIVEEDIT = WM_APP  + 402;  // アクティブなエディットコントロールのハンドルを取得
  TPMSG_SETSBHINTSTR  = WM_APP  + 403;  // ステータスバーのヒントに文字をセットする
  TPMSG_GETEDITMODE   = WM_APP  + 404;  // 編集モードを取得
  TPMSG_GETSELMODE    = WM_APP  + 405;  // 選択モードを取得(-1:非選択, 0:選択, 1:矩形選択)
  TPMSG_SETINIDIR     = WM_APP  + 406;  // 任意のINIフォルダの設定を読み込んで反映
  TPMSG_GETTEXT       = WM_APP  + 410;  // スワップファイル経由でテキストを取得する
  TPMSG_SETTEXT       = WM_APP  + 411;  // スワップファイル経由でテキストを設定する
  TPMSG_GETNUM        = WM_APP  + 412;  // キャレット位置の行番号を取得する
  TPMSG_GETROWCOUNT   = WM_APP  + 413;  // アクティブなエディットコントロールの表示行数を取得する
  TPMSG_GETCOLCOUNT   = WM_APP  + 414;  // アクティブなエディットコントロールの表示桁数を取得する
  TPMSG_GETTOPROW     = WM_APP  + 415;  // アクティブなエディットコントロールの画面上端の行番号を取得する
  TPMSG_GETTOPCOL     = WM_APP  + 416;  // アクティブなエディットコントロールの画面左端の桁番号を取得する
  TPMSG_GETOUTPUTMODE = WM_APP  + 417;  // Outputモードか？を調べる
  TPMSG_SETOUTPUTMODE = WM_APP  + 418;  // Outputモードをセットする(失敗したら0, 成功したら1を返す)
  TPMSG_SETSEL        = WM_APP  + 419;  // EM_SETSELのTeraPad版(WIndows95/98/Meでも制限なし？)

type
  TEditMode = (
               emNone,
               emDef,
               emHtml,
               emPerl,
               emRuby,
               emIni,
               emBat,
               emCpp,
               emJava,
               emVb,
               emHsp,
               emDelphi
               );

var
  // RegisterClipboardFormat('TEditor Box Type'); cf Initialization section
  CF_BOXTEXT: Word;

  // TeraPadのツールから起動した場合は、そのTeraPadのウィンドウハンドル、
  // 単独で起動した場合は NULL を格納する。
  TeraPadWindow: HWND;
  TpAppWindow: HWND;

{■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■}

// TeraPadのメインウィンドウか？
function IsTeraPadWindow(Wnd: HWND): Boolean;

// アクティブなエディットコントロールのハンドルを取得
function TpGetActiveEditWnd(TpWnd: HWND): HWND;

// 編集モードを取得
function TpGetEditMode(TpWnd: HWND): TEditMode;

// TeraPadのテキストサイズを取得
// ※サイズ制限を越えた場合は -1 を返す
function TpGetTextSize(TpWnd: HWND): integer;

// テキストサイズを取得
// ※サイズ制限を越えた場合は -1 を返す
function TpLength(S: string): integer;

// 全てのテキストを取得
function TpGetText(TpWnd: HWND): string;

// 全てのテキストを取得(スワップファイル経由)
function TpGetTextEx(TpWnd: HWND): string;

// 全てのテキストをセット
procedure TpSetText(TpWnd: HWND; Text: string);

// 全てのテキストをセット(スワップファイル経由)
// *元に戻せるタイプ
procedure TpSetTextEx(TpWnd: HWND; Text: string);

// 全てのテキストをセット(スワップファイル経由)
// *元に戻せるないがセット後のスクロールがスムーズ
procedure TpSetTextExLines(TpWnd: HWND; Text: string);

// 全てのテキストを選択
procedure TpSelectAll(TpWnd: HWND);

// テキストの開始位置から終了位置まで選択
procedure TpSelect(TpWnd: HWND; StartIndex, EndIndex: integer);

// 選択テキストを取得
function TpGetSelText(TpWnd: HWND): string;

// 選択テキストを取得(スワップファイル経由)
function TpGetSelTextEx(TpWnd: HWND): string;

// 選択領域にテキストをセット
procedure TpReplaceSel(TpWnd: HWND; Text: string);

// 選択領域にテキストをセット(スワップファイル経由)
procedure TpReplaceSelEx(TpWnd: HWND; Text: string);

// TeraPadの実行ファイルのパスを取得
function TpGetPath(TpWnd: HWND): string;

// TeraPadのバージョンを取得
function TpGetVer(TpWnd: HWND): string;

// TeraPadにファイルを読み込ませる
procedure TpLoadFile(TpWnd: HWND; FilePath: string);

// 上書き保存を実行
procedure TpSaveFile(TpWnd: HWND);

// TeraPadのステータスバーのヒント領域に文字をセット
procedure TpSetHint(TpWnd: HWND; Text: string);

// クリップボードのテキストを取得
// ※TeraPadで矩形選択でコピーしたテキストの場合は true を返す
function TpGetClipboardText(var Text: string): Boolean;

// クリップボードへテキストを送る
//   Text: クリップボードへ送るテキスト
//   Box : 矩形貼り付けする場合は true をセットする
procedure TpSetClipboardText(Text: string; Box: Boolean);

// 選択領域のテキストをクリップボードへコピー
procedure TpCopy(TpWnd: HWND);

// 選択領域のテキストを削除してクリップボードへコピー(切り取り)
procedure TpCut(TpWnd: HWND);

// クリップボードのテキストを貼り付け
procedure TpPaste(TpWnd: HWND);

// 行移動
//   Row : 移動する行
//   Line: 論理行で移動
procedure TpRowJump(TpWnd: HWND; Row: integer; Line: Boolean);

// 文末移動
procedure TpEndJump(TpWnd: HWND);

// INIを読み込んで反映
procedure TpReadIni(TpWnd: HWND);

// TeraPadをアクティブにする
procedure TpActiveWindow(TpWnd: HWND);

// 選択モードを取得
// ※戻り値: -1=非選択, 0=選択, 1=矩形選択
function TpGetSelMode(TpWnd: HWND): integer;

// TeraPadのINIファイルのパスを取得
function TpGetIniPath(TpWnd: HWND): string;

// 任意のINIフォルダの設定を読み込んで反映
procedure TpSetIniDir(TpWnd: HWND; IniDir: string);

// キャレット位置の行番号を取得
// NumMode: TP_NUM_ROW=描画される１行ごとの行番号, TP_NUM_LINE=論理行
function TpGetNum(TpWnd: HWND; NumMode: integer): integer;

//アクティブなエディットコントロールの表示行数を取得する
function TpGetRowCount(TpWnd: HWND): integer;

//アクティブなエディットコントロールの表示桁数を取得する
function TpGetColCount(TpWnd: HWND): integer;

//アクティブなエディットコントロールの画面上端の行番号を取得する
function TpGetTopRow(TpWnd: HWND): integer;

//アクティブなエディットコントロールの画面左端の桁番号を取得する
function TpGetTopCol(TpWnd: HWND): integer;

// ツールバーのウィンドウハンドルを取得
function TpGetToolbarWnd(TpWnd: HWND): HWND;

// TeraPadで編集中のファイルパスを取得
function TpGetFilePath(TpWnd: HWND): string;

// TeraPadがOutputモードか？を調べる
// ※Outputモード時はTrueを返す
function TpGetOutputMode(TpWnd: HWND): Boolean;

// Outputモードをセットする
// ※成功したらtrueを返す
// ※OutputMode=true  :「無題 and 未更新」以外は失敗する
// ※OutputMode=false : Outputモード時以外は失敗する
function TpSetOutputMode(TpWnd: HWND; OutputMode: Boolean): Boolean;

// EM_SETSELのTeraPad版
// ※EM_SETSELと違いTpWndはTeraPadのメインフォームのウィンドウハンドルを指定
procedure TpSetSel(TpWnd: HWND; nStart, nEnd: integer);

{■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■}

implementation

// OSの種類を調べる
// 戻り値
//   Win3.1
//   Win95
//   Win98
//   WinMe
//   WinNT
function GetOs: string;
var
  OSVersionInfo : TOSVersionInfo;
  Platform: string;
begin
  Result := '';
  Platform := '';
  OsVersionInfo.dwOSVersionInfoSize := SizeOf(OSVersionInfo);
  GetVersionEx(OsVersionInfo);

  with OSVersionInfo do
  begin
    case OsVersionInfo.dwPlatformId of
      VER_PLATFORM_WIN32s       : Platform :='Win3.1';
      VER_PLATFORM_WIN32_WINDOWS:
      begin
        case Win32MinorVersion of
          0: Platform :='Win95';
          10: Platform :='Win98';
          90: Platform :='WinMe';
        end;
      end;
      VER_PLATFORM_WIN32_NT: Platform :='WinNT';
    end;
  end;

  Result := Platform;
end;

////////////////////////////////////////////////////////////////////////////////

// TeraPadのメインウィンドウか？
function IsTeraPadWindow(Wnd: HWND): Boolean;
var
  Cls: array[0..255] of Char;
begin
  Result := false;
  if Wnd = HWND(nil) then Exit;

  GetClassName(Wnd, Cls, 255);
  if Cls = 'TTeraPadMainForm' then
    Result := true;
end;

// アクティブなエディットコントロールのハンドルを取得
function TpGetActiveEditWnd(TpWnd: HWND): HWND;
begin
  Result := HWND(nil);
  if IsTeraPadWindow(TpWnd) then
    Result := SendMessage(TpWnd, TPMSG_GETACTIVEEDIT, 0, 0);
end;

// 編集モードを取得
function TpGetEditMode(TpWnd: HWND): TEditMode;
var
  ModeInt: integer;
begin
  Result := emNone;

  if IsTeraPadWindow(TpWnd) then
  begin
    ModeInt := SendMessage(TpWnd, TPMSG_GETEDITMODE, 0, 0);

    case ModeInt of
       1: Result := emHtml;
       2: Result := emPerl;
       3: Result := emIni;
       4: Result := emBat;
       5: Result := emCpp;
       6: Result := emJava;
       7: Result := emVb;
       8: Result := emHsp;
       9: Result := emDelphi;
      10: Result := emRuby;
    else
         Result := emDef;
    end;
  end;
end;

// TeraPadのテキストサイズを取得
// ※サイズ制限を越えた場合は -1 を返す
function TpGetTextSize(TpWnd: HWND): integer;
var
  TpEdWnd: HWND;
begin
  Result := -1;

  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    Result := SendMessage(TpEdWnd, WM_GETTEXTLENGTH, 0, 0);

    // OSが「WinNT」の時は制限なし…
    if GetOs <> 'WinNT' then
    begin
      // WM_GETTEXT で取得できるサイズをオーバーしている場合は -1 を返す
      if Result > MAX_GETTEXT_SIZE then
        Result := -1;
    end;
  end;
end;

// テキストサイズを取得
// ※サイズ制限を越えた場合は -1 を返す
function TpLength(S: string): integer;
begin
  if (GetOs <> 'WinNT') and
     (Length(S) > MAX_GETTEXT_SIZE) then
    Result := -1
  else
    Result := Length(S);
end;

// 全てのテキストを取得
function TpGetText(TpWnd: HWND): string;
var
  TpEdWnd: HWND;
  Size: Longint;
  Text: string;
begin
  Result := '';

  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    Size := TpGetTextSize(TpWnd);

    // サイズオーバーで GetTextSize = -1 の時は '' を返す
    if Size >= 0 then
    begin
      SetLength(Text, Size);
      SendMessage(TpEdWnd, WM_GETTEXT, Size + 1, Longint(PChar(Text)));
      Result := Text;
    end;
  end;
end;

// 全てのテキストをセット
procedure TpSetText(TpWnd: HWND; Text: string);
var
  TpEdWnd: HWND;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    // OSが「WinNT」の時は制限なし…
    if GetOs <> 'WinNT' then
      // オーバーしている場合は切り捨て
      Text := Copy(Text, 1, MAX_GETTEXT_SIZE);

    SendMessage(TpEdWnd, WM_SETTEXT, 0, Longint(PChar(Text)));
  end;
end;

// 全てのテキストを選択
procedure TpSelectAll(TpWnd: HWND);
var
  TpEdWnd: HWND;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    SendMessage(TpEdWnd, EM_SETSEL, 0, -1);
  end;
end;

// テキストの開始位置から終了位置まで選択
procedure TpSelect(TpWnd: HWND; StartIndex, EndIndex: integer);
var
  TpEdWnd: HWND;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    SendMessage(TpEdWnd, EM_SETSEL, StartIndex, EndIndex);
  end;
end;

// 選択テキストを取得
function TpGetSelText(TpWnd: HWND): string;
var
  TpEdWnd: HWND;
  Size: Longint;
  Text: string;
  SelText: string;
  //Sel: DWORD;
  SelStart, SelEnd: DWORD;
begin
  Result := '';

  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    Size := TpGetTextSize(TpWnd);

    // サイズオーバーで GetTextSize = -1 の時は '' を返す
    if Size >= 0 then
    begin
      SetLength(Text, Size);
      SendMessage(TpEdWnd, WM_GETTEXT, Size + 1, Longint(PChar(Text)));

      //Sel := SendMessage(TpEdWnd, EM_GETSEL, 0, 0);
      //SelText := Copy(Text, LoWord(Sel) + 1, HiWord(Sel) - LoWord(Sel));

      SendMessage(TpEdWnd, EM_GETSEL, lParam(@SelStart), wParam(@SelEnd));
      SelText := Copy(Text, SelStart + 1, SelEnd - SelStart);

      Result := SelText;
    end;
  end;
end;

// 選択領域にテキストをセット
procedure TpReplaceSel(TpWnd: HWND; Text: string);
var
  TpEdWnd: HWND;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    // OSが「WinNT」の時は制限なし…
    if GetOs <> 'WinNT' then
      // オーバーしている場合は切り捨て
      Text := Copy(Text, 1, MAX_GETTEXT_SIZE);

    SendMessage(TpEdWnd, EM_REPLACESEL, 0, Longint(PChar(Text)));
  end;
end;

// TeraPadの実行ファイルのパスを取得
function TpGetPath(TpWnd: HWND): string;
var
  PathEditWnd: HWND;
  Buf: PChar;
begin
  Result := '';

  if IsTeraPadWindow(TpWnd) then
  begin
    Buf := StrAlloc(256);
    try
      PathEditWnd := FindWindowEx(TpWnd, 0, 'TTeraPadExeName', nil);

      if PathEditWnd <> HWND(nil) then
      begin
        SendMessage(PathEditWnd, WM_GETTEXT, 256, longint(Buf));
        Result := string(Buf);
      end;
    finally
      StrDispose(Buf);
    end;
  end;
end;

// TeraPadのバージョンを取得
function TpGetVer(TpWnd: HWND): string;
var
  sTMP, sLocale, TpPath: string;
  BufSize: integer;
  pBuf, pInfo: Pointer;
  InfoLen: UINT;
  D: DWORD;
begin
  Result := 'None';

  if IsTeraPadWindow(TpWnd) then
  begin
    TpPath := TpGetPath(TpWnd);

    BufSize := GetFileVersionInfoSize(PChar(TpPath), D);
    GetMem(pBuf, BufSize);
    try
      GetFileVersionInfo(PChar(TpPath), 0, BufSize, pBuf);
      VerQueryValue(pBuf, PChar('\VarFileInfo\Translation'),pInfo, InfoLen);

      if InfoLen > 0 then
        sLocale := IntToHex(Integer(pInfo^), 8)
      else
        Exit;

      sLocale := Copy(sLocale, 5, 4) + Copy(sLocale, 1, 4);

      sTMP := '\StringFileInfo\' + sLocale + '\ProductVersion';
      VerQueryValue(pBuf, PChar(sTMP), pInfo, InfoLen);

      if InfoLen > 0 then
        Result := PChar(pInfo);
    finally
      FreeMem(pBuf);
    end;
  end;
end;

// TeraPadにファイルを読み込ませる
procedure TpLoadFile(TpWnd: HWND; FilePath: string);
var
  TpEdWnd: HWND;
  Modify: Longint;
  Atom: TAtom;
begin
  if IsTeraPadWindow(TpWnd) and (FilePath <> '') then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    Modify := SendMessage(TpEdWnd, EM_GETMODIFY, 0, 0);

    // ■Bool(Modify) = true 時に実行すると確認メッセージを表示する
    // ■Bool(Modify) = true 時に上書き保存する場合はサンプルソース TpSave の方法で…

    //変更されていない場合のみファイルを読み込む
    if not Bool(Modify) then
    begin
      Atom := GlobalAddAtom(@FilePath[1]); // GlobalAtomに書き込む

      if Atom <> 0 then
        SendMessage(TpWnd, TPMSG_LOADFILE, 0, Atom);
    end;
  end;
end;

// 上書き保存を実行
procedure TpSaveFile(TpWnd: HWND);
var
  TpEdWnd: HWND;
  Modify: Longint;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    Modify := SendMessage(TpEdWnd, EM_GETMODIFY, 0, 0);

    //変更されている場合のみ上書き保存
    if Bool(Modify) then
      SendMessage(TpWnd, TPMSG_SAVEFILE, 0, 0);
  end;
end;

// TeraPadのステータスバーのヒント領域に文字をセット
procedure TpSetHint(TpWnd: HWND; Text: string);
var
  Atom: TAtom;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    Atom := GlobalAddAtom(@Text[1]); // GlobalAtomに書き込む

    if Atom <> 0 then
      SendMessage(TpWnd, TPMSG_SETSBHINTSTR, 0, Atom);
  end;
end;

// クリップボードのテキストを取得
// ※TeraPadで矩形選択でコピーしたテキストの場合は true を返す
function TpGetClipboardText(var Text: string): Boolean;
var
  DataStr: THandle;
  DataBox: THandle;
begin
  Result := false;

  {クリップボードが処理中の場合は待つ。。。}
  while (GetOpenClipboardWindow <> HWND(nil)) do;

  OpenClipboard(HWND(nil));
  DataStr := GetClipboardData(CF_TEXT);
  DataBox := GetClipboardData(CF_BOXTEXT);
  try
    if DataStr <> 0 then
    begin
      Text := PChar(GlobalLock(DataStr));

      //矩形選択テキスト？
      if DataBox <> 0 then
        Result := true;
    end
    else
      Text := '';
  finally
    if DataStr <> 0 then GlobalUnlock(DataStr);
    CloseClipboard;
  end;
end;

// クリップボードへテキストを送る
//   Text: クリップボードへ送るテキスト
//   Box : 矩形貼り付けする場合は true をセットする
procedure TpSetClipboardText(Text: string; Box: Boolean);
var
  Data: THandle;
  Size:integer;
  Mem: Pointer;
begin
  {クリップボードが処理中の場合は待つ。。。}
  while (GetOpenClipboardWindow <> HWND(nil)) do;

  Size := Length(Text) + 1;
  try
    Data := GlobalAlloc(GMEM_MOVEABLE, Size);
    OpenClipboard(HWND(nil));
    EmptyClipboard;
    try
      Mem := GlobalLock(Data);
      try
        CopyMemory(Mem, pChar(Text), Size);
        SetClipboardData(CF_TEXT, Data);
        
        if Box then
          SetClipboardData(CF_BOXTEXT, Data);
      finally
        GlobalUnlock(Data);
      end;
    except
      GlobalFree(Data);
    end;
  finally
    CloseClipboard;
  end;
end;

// 選択領域のテキストをクリップボードへコピー
procedure TpCopy(TpWnd: HWND);
var
  TpEdWnd: HWND;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    SendMessage(TpEdWnd, WM_COPY, 0, 0);
  end;
end;

// 選択領域のテキストを削除してクリップボードへコピー(切り取り)
procedure TpCut(TpWnd: HWND);
var
  TpEdWnd: HWND;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    SendMessage(TpEdWnd, WM_CUT, 0, 0);
  end;
end;

// クリップボードのテキストを貼り付け
procedure TpPaste(TpWnd: HWND);
var
  TpEdWnd: HWND;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    SendMessage(TpEdWnd, WM_PASTE, 0, 0);
  end;
end;

// 行移動
//   Row : 移動する行
//   Line: 論理行で移動
procedure TpRowJump(TpWnd: HWND; Row: integer; Line: Boolean);
var
  L: integer;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    if Line then
      L := 1
    else
      L := 0;

    SendMessage(TpWnd, TPMSG_ROWJUMP, Row, L);
  end;
end;

// 文末移動
procedure TpEndJump(TpWnd: HWND);
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    SendMessage(TpWnd, TPMSG_ENDJUNP, 1, 0);
  end;
end;

// INIを読み込んで反映
procedure TpReadIni(TpWnd: HWND);
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    SendMessage(TpWnd, TPMSG_READINI, 0, 0);
  end;
end;

// TeraPadをアクティブにする
procedure TpActiveWindow(TpWnd: HWND);
var
  dwTpTID,
  dwActiveTID: DWORD;
  sp_time: Longint;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    dwActiveTID := GetWindowThreadProcessId(GetForegroundWindow, nil);
    dwTpTID     := GetWindowThreadProcessId(TpWnd, nil);
  
    AttachThreadInput(dwTpTID, dwActiveTID, true);
  
    SystemParametersInfo(SPI_GETFOREGROUNDLOCKTIMEOUT, 0, @sp_time, 0);
    SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT, 0, Pointer(0), 0);
    try
      SetForegroundWindow(TpWnd);
    finally
      SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT, 0, @sp_time, 0);
      AttachThreadInput(dwTpTID, dwActiveTID, false);
    end;
  end;
end;

// 選択モードを取得
// ※戻り値: -1=非選択, 0=選択, 1=矩形選択
function TpGetSelMode(TpWnd: HWND): integer;
begin
  Result := 0;

  if IsTeraPadWindow(TpWnd) then
  begin
    Result := SendMessage(TpWnd, TPMSG_GETSELMODE, 0, 0);
  end;
end;

// TeraPadのINIファイルのパスを取得
function TpGetIniPath(TpWnd: HWND): string;
var
  PathEditWnd: HWND;
  Buf: PChar;
begin
  Result := '';

  if IsTeraPadWindow(TpWnd) then
  begin
    Buf := StrAlloc(256);
    try
      PathEditWnd := FindWindowEx(TpWnd, 0, 'TTeraPadIniFilePath', nil);

      if PathEditWnd <> HWND(nil) then
      begin
        SendMessage(PathEditWnd, WM_GETTEXT, 256, longint(Buf));
        Result := string(Buf);
      end;
    finally
      StrDispose(Buf);
    end;
  end;
end;

// 任意のINIフォルダの設定を読み込んで反映
procedure TpSetIniDir(TpWnd: HWND; IniDir: string);
var
  Atom: TAtom;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    Atom := GlobalAddAtom(@IniDir[1]); // GlobalAtomに書き込む

    if Atom <> 0 then
      SendMessage(TpWnd, TPMSG_SETINIDIR, 0, Atom);
  end;
end;

// キャレット位置の行番号を取得
function TpGetNum(TpWnd: HWND; NumMode: integer): integer;
begin
  Result := SendMessage(TpWnd, TPMSG_GETNUM, NumMode, 0);
end;

//アクティブなエディットコントロールの表示行数を取得する
function TpGetRowCount(TpWnd: HWND): integer;
begin
  Result := SendMessage(TpWnd, TPMSG_GETROWCOUNT, 0, 0);
end;

//アクティブなエディットコントロールの表示桁数を取得する
function TpGetColCount(TpWnd: HWND): integer;
begin
  Result := SendMessage(TpWnd, TPMSG_GETCOLCOUNT, 0, 0);
end;

//アクティブなエディットコントロールの画面上端の行番号を取得する
function TpGetTopRow(TpWnd: HWND): integer;
begin
  Result := SendMessage(TpWnd, TPMSG_GETTOPROW, 0, 0);
end;

//アクティブなエディットコントロールの画面左端の桁番号を取得する
function TpGetTopCol(TpWnd: HWND): integer;
begin
  Result := SendMessage(TpWnd, TPMSG_GETTOPCOL, 0, 0);
end;

// ツールバーのウィンドウハンドルを取得
function TpGetToolbarWnd(TpWnd: HWND): HWND;
begin
  Result := FindWindowEx(TpWnd, 0, 'TToolbar', nil);
end;

// TeraPadで編集中のファイルパスを取得
function TpGetFilePath(TpWnd: HWND): string;
var
  PathEditWnd: HWND;
  Buf: PChar;
begin
  Result := '';

  if IsTeraPadWindow(TpWnd) then
  begin
    Buf := StrAlloc(256);
    try
      PathEditWnd := FindWindowEx(TpWnd, 0, 'TEdit', nil);

      if PathEditWnd <> HWND(nil) then
      begin
        SendMessage(PathEditWnd, WM_GETTEXT, 256, longint(Buf));
        Result := string(Buf);
      end;
    finally
      StrDispose(Buf);
    end;
  end;
end;

// TeraPadがOutputモードか？を調べる
// ※Outputモード時はTrueを返す
function TpGetOutputMode(TpWnd: HWND): Boolean;
begin
  Result := Boolean(SendMessage(TpWnd, TPMSG_GETOUTPUTMODE, 0, 0));
end;

// Outputモードをセットする
// ※成功したらtrueを返す
// ※OutputMode=true  :「無題 and 未更新」以外は失敗する
// ※OutputMode=false : Outputモード時以外は失敗する
function TpSetOutputMode(TpWnd: HWND; OutputMode: Boolean): Boolean;
begin
  if OutputMode then
    Result := Boolean(SendMessage(TpWnd, TPMSG_SETOUTPUTMODE, TP_OUTPUT_ON, 0))
  else
    Result := Boolean(SendMessage(TpWnd, TPMSG_SETOUTPUTMODE, TP_OUTPUT_OFF, 0));
end;

// EM_SETSELのTeraPad版
// ※EM_SETSELと違いTpWndはTeraPadのメインフォームのウィンドウハンドルを指定
procedure TpSetSel(TpWnd: HWND; nStart, nEnd: integer);
begin
  SendMessage(TpWnd, TPMSG_SETSEL, nStart, nEnd);
end;

////////////////////////////////////////////////////////////////////////////////

function GetTextEx(TpWnd: HWND; AllSel: integer): string;
var
  TpEdWnd: HWND;
  dwSize: integer;
  Buf: string;
  MemoryPos: Pointer;
  hMap: THandle;
  S: PChar;
begin
  Result := '';

  if IsTeraPadWindow(TpWnd) then
  begin
    // 取得するテキストのサイズを調べる
    TpEdWnd := TpGetActiveEditWnd(TpWnd);
    dwSize := SendMessage(TpEdWnd, WM_GETTEXTLENGTH, 0, 0) + 1;
    // マップオブジェクトを作成
    Buf := Format('%s%u', [TP_MAP_PREFIX, TpWnd]);
    hMap := CreateFileMapping(INVALID_HANDLE_VALUE, nil, PAGE_READWRITE, 0, dwSize, PChar(Buf));
    if hMap <> 0 then
    begin
      // マップオブジェクトを書き込み可能なメモリ領域に固定する
      MemoryPos := MapViewOfFile(hMap, FILE_MAP_WRITE, 0, 0, 0);
      if MemoryPos <> nil then
      begin
        // メモリ領域を初期化
        ZeroMemory(MemoryPos, dwSize);
        // マップオブジェクトをメモリ領域から解放する
        UnmapViewOfFile(MemoryPos);
        // テキストを取得
        if Boolean(SendMessage(TpWnd, TPMSG_GETTEXT, AllSel, 0)) then
        begin
          // マップオブジェクトを読み取り可能なメモリ領域に固定する
          MemoryPos := MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0);
          if MemoryPos <> nil then
          begin
            // マッピングファイルからデータをコピーする
            S := StrAlloc(dwSize);
            try
              CopyMemory(S, MemoryPos, dwSize);
              // テキストの設定
              Result := string(S);
              // マップオブジェクトをメモリ領域から解放する
              UnmapViewOfFile(MemoryPos);
            finally
              StrDispose(S);
            end;
          end;
        end;
      end;
      // マップオブジェクトを閉じる
      CloseHandle(hMap);
    end;
  end;
end;

// 全てのテキストを取得(スワップファイル経由)
function TpGetTextEx(TpWnd: HWND): string;
begin
  Result := GetTextEx(TpWnd, TP_TEXT_ALL);
end;

// 選択テキストを取得(スワップファイル経由)
function TpGetSelTextEx(TpWnd: HWND): string;
begin
  Result := GetTextEx(TpWnd, TP_TEXT_SEL);
end;

procedure SetTextEx(TpWnd: HWND; Text: string; AllSel: integer);
var
  dwSize: integer;
  Buf: string;
  MemoryPos: Pointer;
  hMap: THandle;
  S: PChar;
begin
  if IsTeraPadWindow(TpWnd) then
  begin
    // 設定するテキストのサイズを調べる
    dwSize := Length(Text) + 1;
    // マップオブジェクトを作成
    Buf := Format('%s%u', [TP_MAP_PREFIX, TpWnd]);
    hMap := CreateFileMapping(INVALID_HANDLE_VALUE, nil, PAGE_READWRITE, 0, dwSize, PChar(Buf));
    if hMap <> 0 then
    begin
      // マップオブジェクトを書き込み可能なメモリ領域に固定する
      MemoryPos := MapViewOfFile(hMap, FILE_MAP_WRITE, 0, 0, 0);
      if MemoryPos <> nil then
      begin
        // メモリ領域にテキストをコピー
        S := StrAlloc(dwSize);
        S := StrCopy(S, PChar(Text));
        try
          CopyMemory(MemoryPos, S, dwSize);
        finally
          StrDispose(S);
        end;
        // マップオブジェクトをメモリ領域から解放する
        UnmapViewOfFile(MemoryPos);
        // テキストを設定
        SendMessage(TpWnd, TPMSG_SETTEXT, AllSel, dwSize);
      end;
      // マップオブジェクトを閉じる
      CloseHandle(hMap);
    end;
  end;
end;

// 全てのテキストをセット(スワップファイル経由)
// *元に戻せるタイプ
procedure TpSetTextEx(TpWnd: HWND; Text: string);
begin
  SetTextEx(TpWnd, Text, TP_TEXT_ALL);
end;

// 全てのテキストをセット(スワップファイル経由)
// *元に戻せるないがセット後のスクロールがスムーズ
procedure TpSetTextExLines(TpWnd: HWND; Text: string);
begin
  SetTextEx(TpWnd, Text, TP_TEXT_LINES);
end;

// 選択領域にテキストをセット(スワップファイル経由)
procedure TpReplaceSelEx(TpWnd: HWND; Text: string);
begin
  SetTextEx(TpWnd, Text, TP_TEXT_SEL);
end;

////////////////////////////////////////////////////////////////////////////////

initialization
  CF_BOXTEXT := RegisterClipboardFormat('TEditor Box Type');

  TpAppWindow := HWND(nil);
  TeraPadWindow := GetForegroundWindow;
  if Not IsTeraPadWindow(TeraPadWindow) then
    TeraPadWindow := HWND(nil)
  else
    TpAppWindow := GetWindow(TeraPadWindow, GW_OWNER);

end.
